@@ -1,4 +1,4 @@
-# $Id: FSM.pm,v 1.7 2013/01/06 23:06:57 Paulo Exp $
+# $Id: FSM.pm,v 1.8 2013/05/28 22:46:52 Paulo Exp $
 
 package Parse::FSM;
 
@@ -20,7 +20,7 @@ use Data::Dump 'dump';
 use Text::Template 'fill_in_string';
 use File::Slurp;
 
-our $VERSION = '1.06';
+our $VERSION = '1.07';
 
 #------------------------------------------------------------------------------
 
@@ -398,7 +398,8 @@ sub _compute_fsm {
 		
 		# check all states in turn
 		for my $state (@{$self->_state_table}) {
-			while (my($token, $next_state) = each %$state) {
+			my %state_copy = %$state;
+			while (my($token, $next_state) = each %state_copy) {
 				next unless my($subrule_name, $quant) = 
 						$token =~ /^ \[ (.*) \] ( [?*] )? $/x;
 
@@ -637,7 +638,7 @@ sub write_module {
 #------------------------------------------------------------------------------
 # template code for grammmar parser
 my $TEMPLATE = <<'END_TEMPLATE';
-# $Id: FSM.pm,v 1.7 2013/01/06 23:06:57 Paulo Exp $
+# $Id: FSM.pm,v 1.8 2013/05/28 22:46:52 Paulo Exp $
 # Parser generated by Parse::FSM
 
 package # hide from CPAN indexer
